package net.rmi.rjs.fc.ssinstaller;

import java.io.*;
import java.util.*;
import utils.OsUtils;

public class SSUnixInstaller{

private String userHome;
private String ssHome;
private File outputJarFile = null;
private String progress;
private String scriptName = "grant.priv";
private Process p;

public SSUnixInstaller(){

	userHome = SSInstallerUtil.getUserHome();
}


public  boolean isFedora(String name){

	if (name.indexOf("Fedora") > -1)
		return true;
	else
		return false;

}

public String createSSHome(){
	
	String ssloc =  SSInstallerUtil.getUserHome() + File.separator 
					+ SSInstallerUtil.getSSHome();
	
	ssHome = SSInstallerUtil.getUserHome() + File.separator 
			+ SSInstallerUtil.getSSHome() + File.separator 
			+ SSInstallerUtil.getSSName();
	try{
		SSInstallerUtil.createDir(ssloc);
		SSInstallerUtil.createDir(ssHome);
	}catch (Exception e){
		return "Error creating ss home: " + e.toString();
	}
	return "";
}

public String beamOverFiles(){
	
	//Beam over jar file
	outputJarFile = new File (ssHome + File.separator + SSInstallerUtil.getJarName() + ".jar");
	String urlString = SSInstallerUtil.getSSUnixUrl();
	try{
		SSInstallerUtil.downloadScreenSaverJar(outputJarFile, urlString);
	}catch(Exception e){
		
		return "Error Downloading Jar File: " + e.toString();
	}
	
	return "";
}

public String uncompressFiles(){
	//	uncompress jar
	try{
		SSInstallerUtil.uncompressScreenSaverJar(outputJarFile);
	}catch(Exception e){
	
		return "Error uncompressing Jar File: " + e.toString();
	}

	return "";
}

public String install(){

	String isDone = "";
	progress = "... Starting installation" +
			  "\n... OS= " + OsUtils.getOsName();
		
	progress = progress + 
			  "\n... Reading Properties file";
	//isDone = SSInstallerUtil.readPropertiesfile();
	isDone = SSInstallerUtil.setProperties();
	if (isDone != "")
	{
		return isDone;
	}
	
	isDone = createSSHome();
	if (isDone != "")
	{
		return isDone;
	}
	progress = progress + 
		"\n... Creating SS home at "+ ssHome + "... completed";
	
	isDone = beamOverFiles();
	if (isDone != "")
	{
		return isDone;
	}
	progress = progress + 
		"\n... Beaming files over to: " + ssHome + "... completed";
	
	
	isDone = uncompressFiles();
	if (isDone != "")
	{
		return isDone;
	}
	progress = progress + 
		"\n... Uncompressing files... Completed";
	
	
	isDone = changeFilePrivs();
	if (isDone != "")
	{
		return isDone;
	}
	progress = progress + 
		"\n... Granting executable privilege... completed";

	
	isDone = addScreensaverToConfgFile();
	if (isDone != "")
	{
		return isDone;
	}
	progress = progress + 
		"\n... Modifying .xscreensaver... completed";
	
	
	progress = progress + 
		"\n... Installation Completed ";
	
	return "";	
}

public String getProgress(){
	
	return progress;
}

public String changeFilePrivs(){

	String[] pc = new String[2];
	int exitValue = 0;
	//Grant execute permissions
	
	try{
		pc[0]= "sh";
		pc[1]= ssHome + File.separator + scriptName;
		
		p = Runtime.getRuntime().exec(pc);
		exitValue = p.waitFor();
		
	}catch(Exception e){

		return "Error granting privs Exit Value: " + exitValue + e.toString();
	}
	
	if (exitValue != 0)
		return "Error granting privs Exit Value: " + exitValue;
	
	
	return "";
}

public String addScreensaverToConfgFile(){

	//get file xscreensaver from the user home directory
	File confgFile = new File(SSInstallerUtil.getUserHome() + "/" + ".xscreensaver");
	
	if (!confgFile.exists())
	{
		return "Error: .xscreensaver file does not exist at user home. /n" +
				"Please run xscreensaver-demo and try runing installation again.";
	}
	//get java home
	String jhome = SSInstallerUtil.getJavaHome();
	
	//Check that this is jdk 1.5
	int jindx1 = jhome.indexOf("jdk");
	int jindx2 = jhome.indexOf("1.5");
	if (jindx1 == -1 || jindx2 == -1)
		System.out.println("The screensaver may not run correctly without the jdk 1.5.");

	//Remove the jre word from the java home.
	jindx1 = jhome.indexOf("jre");
	if (jindx1 > -1)
		jhome = jhome.substring(0, jindx1 - 1);

	String sline = "";
	int sloc = -1;

 	try {
	        //File fin = new File(filePath);
	        RandomAccessFile raf = new RandomAccessFile(confgFile, "rw");

			//jif.println("File Length " + confgFile.length());

	       	//read first line
	        sline = raf.readLine();
	        ArrayList lines = new ArrayList();

	        while (sline != null){
				// Time for idel time before the ss starts
				sloc = sline.indexOf("timeout:");
				if (sloc > -1){
					lines.add("timeout:\t0:01:00");
					sline = raf.readLine();
					continue;
				}
				//cycle time: time the ss should live before restaring
				sloc = sline.indexOf("cycle:");
				if (sloc > -1){
					lines.add("cycle:\t12:00:00");
					sline = raf.readLine();
					continue;
				}
				//mode: one -> one screensaver working
				sloc = sline.indexOf("mode:");
				if (sloc > -1){
					lines.add("mode:\tone");
					sline = raf.readLine();
					continue;
				}
				//Selected: Bouncing line is screensaver 0.
				sloc = sline.indexOf("selected:");
								if (sloc > -1){
									lines.add("selected:\t0");
									sline = raf.readLine();
									continue;
				}


				//Check that this screensaver does not exist in the file, if so, do not include the line.
				sloc = sline.indexOf(SSInstallerUtil.getSSName());
				if (sloc > -1){
					sline = raf.readLine();
					continue;
				}

				//Add line to the ArrayList
				lines.add(sline);

				//Check for the word "programs" in the file. Add screensaver Bouncingline to the next line.
	        	sloc = sline.indexOf("programs:");

	        	//Once it finds the string "programs:" then insert the line with the SS info into the List.
	        	if (sloc > -1){
					lines.add("\t\" " + SSInstallerUtil.getSSName() + " (java)\"" + ssHome  
									  + File.separator + SSInstallerUtil.getSSName() + " -root -jdkhome " + jhome +" \\n\\");
				}

				//read next line
				sline = raf.readLine();

			}

			// clear out the file
			raf.setLength(0);
			raf.seek(0);
			System.out.println("lines = " + lines.size());

			//rewrite the file
			for (int i=0; i < lines.size(); i++){

				sline = (String)lines.get(i);
				raf.writeBytes(sline + "\n");
			}

	        raf.close();
	    } catch (IOException e) {
	    	return "Error reading and writting .xscreensaver" + e.toString();
	    }
	    
	    return "";
}

}
